#ifndef _CARD_BASE_HPP_
#define _CARD_BASE_HPP_

#include <vector>
#include <memory>
#include <atomic>
#include "point.h"
#include "common.h"

#define CARD_LOST_TIME_OUT (60*1000)

struct task;
template<typename T> struct zloop;
struct select_tool;
struct smooth_tool;
struct monkey_person;
struct card_message_handle;
struct card_location_base;
struct message_locinfo;
struct loc_message;
struct mine_tool;
struct location_card;
struct card_pos;
struct area_hover;
struct site_area_hover;
struct site;
struct area_tool;

namespace YA{struct _CARD_POS_;}

struct card:point
{
	card(uint32_t id,uint16_t dis,uint64_t type,int32_t deptid,int32_t level_id,uint32_t cid)
		:m_type(type)
		,m_time(0)
		,m_id(id)
		,m_cid(cid)
		,m_deptid(deptid)
		,m_level_id(level_id)
		,m_stat(0)
		,m_biz_stat(0)
		,m_pwr_stat(0)
		,m_display(dis)
		,m_ct(0)
        ,m_acc(0)
		,m_speed(0)
	{}
	uint64_t type_(){return m_type;}
	uint64_t time_(){return m_time;}

	uint64_t m_type;			//类型
    uint64_t m_time;			//时间戳 ms
	uint32_t m_id;				//卡号
	uint32_t m_cid;				//标识id 人staff_id 车 vehicle_id
	int32_t  m_deptid;			//部门编号
	int32_t  m_level_id;		//职务级别
	int		 m_stat;			//运动静止状态
	int		 m_biz_stat;
	int	     m_pwr_stat;		//电量状态
	uint16_t m_display;			//1显示0不显示，往前端推送
	uint16_t m_ct;				//ct
    ///人卡(加速度状态)，其值就是0和1，掘进机和采煤机，其值就是0~255，车辆，其值就需要乘以0.01，用于表示加速度值
    double   m_acc;
	double   m_speed;			//速度
};

struct card_location_base:card,std::enable_shared_from_this<card_location_base>
{
	uint64_t m_timeval=0;
	uint16_t m_display_old{0};			//1显示0不显示，往前端推送
    void update_display(){m_display_old=m_display;}
    bool eq_display(){return m_display_old==m_display;}
    std::uint8_t m_event[CARD_EVENT_COUNT_MAX]{0};
    std::unique_ptr<select_tool> m_sel_tool;
    std::unique_ptr<smooth_tool> m_smo_tool;
	std::unique_ptr<card_message_handle> m_message_handle;
	std::unique_ptr<location_card> m_his_location_card;

	std::atomic<int> m_upmine_flag{0};

	time_t m_help_last_time=0;
	int    m_help_bit=0;


    card_location_base()=default;
    card_location_base(const std::string&type,uint32_t id,uint16_t dis,int16_t t,int32_t,int32_t,uint32_t );

	void inc_upmine_flag(int flag){m_upmine_flag=flag;}
    int upmine_flag(){return m_upmine_flag.load();}

	virtual void do_business(const std::shared_ptr<site>&site,const point &pt,double acc)=0;
	virtual void on_timer()=0;
    virtual std::shared_ptr<mine_tool> get_mine_tool()=0;
    virtual void clear();
	virtual void reset(std::shared_ptr<monkey_person> mp){}
    virtual int get_vehicle_type_id(){return  0;}
	virtual void set_area_info(int mapid,double scale,int areaid,uint64_t t,int type)=0;
	virtual int get_area(){return -1;}
    virtual std::shared_ptr<area_tool> get_area_tool()=0;
    virtual int get_vehicle_category_id(){return -1;}
    virtual int get_workline(){return -1;}
	virtual void get_card(bool f)=0;
	virtual void site_hover(int sid){}

    void make_his_location(uint64_t t,const point & pt,bool bclose = false);
    void on_message(zloop<task*> * loop,const message_locinfo&loc,bool is_history);
    void on_location(const std::vector<point>&vp,const std::vector<loc_message> &lm );
    void do_status(int st);
	void upt_card_pos(YA::_CARD_POS_&cp, point &pt);
	void del_card_pos();
	int	 get_stat();
	void put_three_rates(card_pos &);
    void set_event_flag(EVENT_TYPE et,std::uint8_t f=1){m_event[et]=f;}
    bool get_event_flag(EVENT_TYPE et){return m_event[et];}

    bool is_person() const;
    bool is_vehicle() const;
    void set_base_data(uint32_t cid,uint16_t type,uint32_t deptid,int32_t level_id);
	virtual ~card_location_base();

    static std::shared_ptr<card_location_base> make_person(const std::string&type,uint32_t cardid,uint16_t needdisplay,int16_t t,
			int32_t deptid,int32_t level_id,uint32_t cid,int wl,const std::string & sname,const std::string & dname,int worktype_id);

    static std::shared_ptr<card_location_base> make_car(const std::string&type,uint32_t cardid,uint16_t needdisplay,int16_t t,
			int32_t deptid, int32_t categoryid, int type_id,int32_t level_id,uint32_t cid);
};

#endif

